%% Greenhouse Climate Control
%
% Greenhouse Optimal Climate Control, a problem with external inputs
%
%% Problem description
% Taken from the book: Optimal Control of Greenhouse Cultivation
% G. van Straten, L.G. van Willigenburg, R.J.C. van Ooteghem, E. van Henten
%
% ISBN: 9781420059618
% CRC Pr I Llc Books
%
% Programmers:
% Gerard Van Willigenburg (Wageningen University)

% Copyright (c) 2009-2009 by Tomlab Optimization Inc.

%% Problem setup

% Define tomSym variable t (time) and t_f (fixed final time)
close all; toms t; t_f = 48;

% Define and set time axis
p = tomPhase('p', t, 0, t_f, 120);
setPhase(p);

% Define the state and control variables
tomStates x1 x2 x3
tomControls u
x = [x1; x2; x3];

% Initial state
xi = [0; 10; 0];

% Initial guess
x0 = {icollocate(x == xi); collocate(u == 0)};

% Boundary conditions
cbnd = initial(x == xi);

% Parameters
pW = 3e-6/40; pT = 1; pH = 0.1;
pHc = 7.5e-2/220;
pWc = 3e4/220;

% External inputs: [time, sunlight, outside temperature]
% sunlight has minimum zero
te  = (-1:0.2:49)';
tue = [te max(800*sin(4*pi*te/t_f-0.65*pi),0) 15+10*sin(4*pi*te/t_f-0.65*pi)];

% Extract external inputs from table tue through interpolation
ue1 = interp1(tue(:,1),tue(:,2),t);
ue2 = interp1(tue(:,1),tue(:,3),t);

%Differential equations
ceq = collocate({
    dot(x1) == pW*ue1*x2
    dot(x2) == pT*(ue2-x2)+pH*u;
    dot(x3) == pHc*u});

% Control bounds
cbox = {0 <= collocate(u) <= 100};

% Cost function to be minimized
objective = final(x3-pWc*x1);

%% Solve the problem
options = struct;
options.name = 'Greenhouse Problem';
[solution, result] = ezsolve(objective, {cbox, cbnd, ceq}, x0, options);

% Obtain final solution t,x1,...,u,..
% that overwrite the associated tomSym variables
t  = subs(collocate(t),solution);
x1 = subs(collocate(x1),solution);
x2 = subs(collocate(x2),solution);
x3 = subs(collocate(x3),solution);
u = subs(collocate(u),solution);
J = subs(objective,solution);

%Plot external inputs and control
if ~exist('plf','var') || plf
  figure(1);
  plot(tue(:,1),tue(:,2)/40,tue(:,1),tue(:,3),t,u/5,t,x2); axis([0 t_f -1 35]);
  xlabel('Time [h]');
  ylabel('Heat input/5, temperatures & light/40');
  legend('Light/40 [W]','Outside temp. [oC]','Heat input/5 [W],...','Greenhouse temp. [oC]');
  title('Optimal heating, outside temperature and light');

  % Plot the optimal state
  figure(2)
  sf1=1200; sf3=60;
  plot(t,[sf1*x1 x2 sf3*x3]); axis([0 t_f -5 40]);
  xlabel('Time [h]'); ylabel('states');
  legend('1200*Dry weight [kg]','Greenhouse temp. [oC]','60*Integral(pHc*u dt) [J]');
  title('Optimal system behavior and the running costs');
end