%% Room temperature control
%
%% Problem Description
% Temperature control from 0.00 to 7.00 hours at night. Finds best heating
% policy at night that brings the temperature back to 20 [oC] in the
% morning irrespective of night temperatures.
%
% Programmers:
% Gerard Van Willigenburg (Wageningen University)

% Copyright (c) 2007-2010 by Tomlab Optimization Inc.

%% Problem setup
% Define tomSym variable t (time)
% and t_f (final time) if the final time is free
close all; toms t; t_f=7; % Fixed final time

% Define & set time axis
p = tomPhase('p', t, 0, t_f, 80);
setPhase(p);
    
% Define the state and control variables
tomStates x; tomControls u

% Initial state
xi=20;

% Initial guess
x0 = {icollocate({x == xi(1)})
      collocate({u == 0})};

% Boundary conditions
cbnd = {initial({x == xi}); final({x == xi})};

% Equality constraints: state-space differential equations
tau=2; pH=0.002; % Parameters

% External input d1
d1=10;
%d1=15-10*sin(pi*t/t_f);

%Differential equation
ceq = collocate({dot(x) == 1/tau*(d1-x) + pH*u});

% Inequality constraints
cbox = {0 <= collocate(u) <= 3600; icollocate(x) >= -Inf};

% Cost function to be minimized
objective = integrate(u+1e-6*dot(u)^2);

% Solve the problem after specifying its name
options = struct;
options.name = 'Temperature control at night';
solution = ezsolve(objective, {cbox, cbnd, ceq}, x0, options);

% Plot intermediate solution
figure; subplot(2,1,1);
ezplot(x); legend('x');
title('State');

subplot(2,1,2);
ezplot(u); legend('u');
title('Optimal control'); drawnow;

% Obtain intermediate solution to initialize the next
xopt = subs(x,solution);
uopt = subs(u,solution);

% Obtain final solution t,x,...,u,..
% that overwrite the associated tomSym variables
t = subs(collocate(t),solution);
x = subs(collocate(x),solution);
u = subs(collocate(u),solution);

%Plot results
figure; plot(t,x,t,u/100); axis([0 t_f -1 50]);
xlabel('Time [h]'); ylabel('Heat input, Inside & Outside temperature');
title('Optimal heating, outside temperature');
legend('Inside temp. [oC]','Outside temp. [oC]');